// Produce index of Microsoft Test Dialog File
//
/* $Log$
*/

// Donated as Freeware by:
//
//			Michael Bate
//			104 Glezen Lane
//			Wayland, MA 01778
//			(508)358-2554
// Compuserve ID: 72077, 1134

// This is a simple program to read a Dialog file (usual extent is .TDL)
// and produce a list of dialog names within it.

// This should help you keep things straight while coding
// Microsoft Test scripts to test your application against
// saved dialog files.

// Invoke via:
//						mstdlg [-c] [-h] filename
//
// If you do not supply an extent, ".TDL" will be assumed.
// The two optional flags are:
//		-c		Precede each output line with "'", the TestBasic comment character.
//				This enables you to paste the output into your TestBasic script,
//				as comments.
//		-h		Precede the output with the name and timestamp of the file.

// Note that I have no inside information about the format of these files.
// I figured out what I needed to know by looking at HEX dumps.
// This program works for me, but there is no assurance that it will
// work for all .TDL files.  It is, of course, possible that a future
// version of Microsoft Test will use a different file format.

// If anyone comes up with useful additions to this program, I hope they
// will let me know.

// This source compiles with Borland C++ 3.1 (and probably earlier versions
// as well).

#include <iostream.h>
#include <fstream.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <time.h>
#include <iomanip.h>
#include <string.h>
#include <dir.h>
#include <stdlib.h>
#include <io.h>

static void usage(void);

struct file_header				// Start of Dialog file
{
	char junk[5];					// I don't know what is stored here.
	int no_dialogs;				// Number of dialogs in the file
	long header_offset;			// Table of dialog names begins at this
										// offset from the start of the file
} header;

struct dialog_ptr					// Each entry in table of dialog names has
{										// this structure.
	int junk[2];					// I don't know what is stored here.
	int n_items;					// Number of items captured
	int if_full;					// LSB seems to indicate FULL comparison
	char name[20];					// Name of the dialog, zero-terminated.
										// names don't actually get this long.
} pt_dialog;

void main(int argc, char **argv)
{
	char fname[MAXPATH];
	int i, header_flag, file_handle;
	char *start_comment;
	struct ftime ftimep;
	struct tm xtime = {0, 0, 0, 0, 0, 0, 0, 0, 0};
	unsigned xread, xfound;

	if ((argc < 2) || (argc > 4))
		usage();
	if (strcmp(argv[1], "/?") == 0)
		usage();

	header_flag = 0;
	start_comment = "";
	i = 1;
	while (argv[i][0] == '-')
	{
		if ((strchr(argv[i], 'h') != NULL) || (strchr(argv[i], 'H') != NULL))
			header_flag = 1;
		if ((strchr(argv[i], 'c') != NULL) || (strchr(argv[i], 'C') != NULL))
			start_comment = "'   ";
		i++;
	}

	strcpy(fname, argv[i]);
	if (strchr(fname, '.') == NULL)
		strcat(fname, ".TDL");

	if ((file_handle = open(fname, O_BINARY | O_RDONLY)) <= 0)
	{
		cout << "Cannot open Dialog file " << fname << '\n';
		exit(2);
	}
	if (header_flag != 0)
	{
		if (getftime(file_handle, &ftimep) != 0)
		{
			cout << "Cannot get timestamp of " << fname << '\n';
			exit(2);
		}
		xtime.tm_sec = ftimep.ft_tsec << 1;
		xtime.tm_min = ftimep.ft_min;
		xtime.tm_hour = ftimep.ft_hour;
		xtime.tm_mday = ftimep.ft_day;
		xtime.tm_mon = ftimep.ft_month - 1;
		xtime.tm_year = ftimep.ft_year + 80;

		(void) mktime(&xtime);

		cout << start_comment << fname << " " << asctime(&xtime)
			<< start_comment << '\n';
	}
	xread = sizeof(header);
	xfound = read(file_handle, &header, sizeof(header));
	if (xfound != xread)
//	if (read(file_handle, &header, sizeof(header)) != sizeof(header))
	{
		cout << "Cannot read file " << fname << '\n';
		exit(2);
	}
	if (lseek(file_handle, header.header_offset, SEEK_SET) < 0)
	{
		cout << "Clobbered file " << fname << '\n';
		exit(2);
	}

	for (i = 1; i <= header.no_dialogs; i++)
	{
		if (read(file_handle, &pt_dialog, sizeof(pt_dialog)) != sizeof(pt_dialog))
		{
			cout << "Cannot read file " << fname << '\n';
			exit(2);
		}
		cout << start_comment << setw(4) << i << " "
			<< setiosflags(ios::left) << setw(20) << pt_dialog.name
			<< resetiosflags(ios::left)
			<< setw(4) << pt_dialog.n_items
			<< (pt_dialog.if_full ? " Full" : " Partial")
			<< '\n';
	}
	close(file_handle);
}

static void usage(void)
{
	cout << "\nUsage:\n\n";
	cout << "      mstdlg [-c] [-h] filename\n\n";
	cout << "filename is the name of a Microsoft Test Dialog file.\n";
	cout << "if extent is not given, \".TDL\" is assumed.\n";
	cout << "  -c:  Begin every output line with TestBasic comment char (\').\n";
	cout << "  -h:  Start with header info (file name and date).\n\n";
	exit(0);
}